#!/usr/local/bin/perl
use strict;
use warnings;

use feature qw(state);
use ReadOscars qw(read_oscars);
use DBI;

my %DBI_OPTS = ( PrintError => 0, RaiseError => 1 );

my %ref_to_year;
my ( $oscar_ref, $movie_ref ) = read_oscars( 'oscars.txt' );
for my $year ( keys %$oscar_ref )
{
  for my $category ( keys %{ $oscar_ref->{$year} } )
  {
    my $ref = $oscar_ref->{$year}{$category};
    $ref_to_year{ $ref } = $year;    # Keys are stringified
  }
}

open_database();
init_database( <DATA> );

for my $movie ( keys %$movie_ref )
{
  for my $category ( keys %{ $movie_ref->{$movie} } )
  {
    my $ref = $movie_ref->{$movie}{$category};
    my $year = $ref_to_year{$ref} or die $movie;

    my $movie_id = insert_movie( $movie );
    for my $name ( @$ref )
    {
      my $person_id = insert_person( $name );
      insert_oscar( $year, $category, $person_id, $movie_id );
    }
  }
}

{
  my $dbh;

  sub open_database
  {
    my $password = get_password();
    $dbh = DBI->connect( 'dbi:mysql:host=localhost', $ENV{USER}, $password, \%DBI_OPTS );
  }

  sub init_database
  {
    chomp( my @lines = @_ );
    for my $statement ( split /;/, join '', @lines )
    {
      $dbh->do( $statement );
    }
  }

  my %movie_id;

  sub insert_movie
  {
    my $title = shift;

    $movie_id{$title} and return $movie_id{$title};
    state $creative_id = 0;
    state $movie_id = 0;
    $_++ for ( $creative_id, $movie_id );
    my $sql = <<"EOSQL";
  INSERT INTO creative_work ( id, title ) VALUES ( ?, ? );
EOSQL
    $dbh->do( $sql, undef, $creative_id, $title );
  $sql = <<'EOSQL';
  INSERT INTO movie ( id, creative_work_id  ) VALUES ( ?, ? );
EOSQL
    $dbh->do( $sql, undef, $movie_id, $creative_id );
    return $movie_id{$title} = $movie_id;
  }

  sub insert_person
  {
    my $name = shift;

    state $person_id = 0;
    $person_id++;
    my $sql = <<"EOSQL";
  INSERT INTO person ( id, name ) VALUES ( ?, ? );
EOSQL
    $dbh->do( $sql, undef, $person_id, $name );
    return $person_id;
  }

  sub insert_oscar
  {
    my ($year, $category, $person_id, $movie_id) = @_;

    state $oscar_id = 0;
    $oscar_id++;
    my $sql = <<"EOSQL";
  INSERT INTO oscar ( id, year, category, for_movie_id ) VALUES ( ?, ?, ?, ? );
EOSQL
    $dbh->do( $sql, undef, $oscar_id, $year, $category, $movie_id  );
    $sql = <<"EOSQL";
  INSERT INTO winner_oscar_assoc ( oscar_id, winner_person_id ) VALUES ( ?, ? );
EOSQL
    $dbh->do( $sql, undef, $oscar_id, $person_id );
    $sql = <<"EOSQL";
  INSERT INTO actor_movie_assoc ( movie_id, person_id ) VALUES ( ?, ? )
EOSQL
    $dbh->do( $sql, undef, $movie_id, $person_id );
  }
}

sub get_password
{
  open my $fh, '<', 'mysql_password' or die $!;
  chomp( my $line = <$fh> );
  return $line;
}

__END__
DROP DATABASE IF EXISTS Lesson8;
CREATE DATABASE Lesson8;
use Lesson8;
CREATE TABLE creative_work ( id MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                             title VARCHAR(100) NOT NULL, genre SET('Science Fiction','Romance','Drama','Biography','Horror','Western') );

CREATE TABLE book ( id MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
       	            author_id MEDIUMINT, creative_work_id MEDIUMINT NOT NULL );

CREATE TABLE movie ( id MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                     creative_work_id MEDIUMINT NOT NULL, based_on_book_id MEDIUMINT );

CREATE TABLE actor_movie_assoc( id MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                                movie_id MEDIUMINT NOT NULL, role VARCHAR(100) NOT NULL, person_id MEDIUMINT NOT NULL );

CREATE TABLE oscar ( id MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                     year SMALLINT NOT NULL, for_movie_id MEDIUMINT NOT NULL,
                     category SET('Best Picture','Actor','Actress','Writer','Editing','Directing','Writing (Adapted)','Writing (Original)') NOT NULL );

CREATE TABLE winner_oscar_assoc ( id MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                                  oscar_id MEDIUMINT NOT NULL, winner_person_id MEDIUMINT NOT NULL );

CREATE TABLE person ( id MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                      name VARCHAR(50) );

